==> webkit2gtk_2.43.3.orig.tar.xz: webkitgtk-2.43.3/Tools/TestWebKitAPI/Tests/IPC/MessageSenderTests.cpp <==
/*
 * Copyright (C) 2022 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "MessageSender.h"

#include "IPCTestUtilities.h"
#include "MessageSenderInlines.h"
#include "Test.h"

namespace TestWebKitAPI {

class MessageSenderTest : public ConnectionTestABBA {
};

namespace {

class SimpleMessageSender : public IPC::MessageSender {
public:
    SimpleMessageSender(IPC::Connection* connection)
        : m_connection(connection)
    {
    }

private:
    IPC::Connection* messageSenderConnection() const { return m_connection; };
    uint64_t messageSenderDestinationID() const { return 0u; };
    IPC::Connection* m_connection;
};

}

// Tests that async reply messages sent to a connection after invalidate()
// will receive the replies as cancelled.
TEST_P(MessageSenderTest, SendAsyncAfterInvalidateCancelsAllAsyncReplies)
{
    ASSERT_TRUE(openBoth());
    b()->invalidate();

    // This works for IPC::Connection.
    {
        HashSet<uint64_t> replies;
        for (uint64_t i = 100u; i < 160u; ++i) {
            b()->sendWithAsyncReply(MockTestMessageWithAsyncReply1 { }, [&, j = i] (uint64_t value) {
                EXPECT_EQ(value, 0u) << j;
                if (!value)
                    replies.add(j);
            }, i);
        }
        while (replies.size() < 60u)
            RunLoop::current().cycle();
    }

    // Same should work via IPC::MessageSender.
    {
        SimpleMessageSender sender { b() };
        HashSet<uint64_t> replies;
        for (uint64_t i = 100u; i < 160u; ++i) {
            sender.sendWithAsyncReply(MockTestMessageWithAsyncReply1 { }, [&, j = i] (uint64_t value) {
                EXPECT_EQ(value, 0u) << j;
                if (!value)
                    replies.add(j);
            }, i);
        }
        while (replies.size() < 60u)
            RunLoop::current().cycle();
    }
}

// Tests that async reply messages sent to a connection after invalidate()
// will receive the error.
TEST_P(MessageSenderTest, SendWithPromisedReply)
{
    ASSERT_TRUE(openBoth());
    b()->invalidate();

    bool done = false;

    b()->sendWithPromisedReply(MockTestMessageWithAsyncReply1 { }, 100)->whenSettled(RunLoop::current(), [&] (MockTestMessageWithAsyncReply1::Promise::Result result) {
        EXPECT_FALSE(result.has_value());
        EXPECT_EQ(result.error(), IPC::Error::InvalidConnection);
        done = true;
    });

    while (!done)
        RunLoop::current().cycle();
}

// Tests that async reply messages sent to a connection after invalidate()
// will receive the error asynchronously.
TEST_P(MessageSenderTest, SendAsyncWithErrorAsynchronousCallback)
{
    ASSERT_TRUE(openBoth());
    b()->invalidate();


    bool done = false;
    b()->sendWithAsyncReply(MockTestMessageWithAsyncReply1 { }, [&] (uint64_t value) {
        // Value is default constructed here, and no error is actually returned.
        done = true;
    }, 100);
    EXPECT_FALSE(done);

    while (!done)
        RunLoop::current().cycle();

    done = false;
    b()->sendWithPromisedReply(MockTestMessageWithAsyncReply1 { }, 100)->whenSettled(RunLoop::current(), [&] (MockTestMessageWithAsyncReply1::Promise::Result result) {
        EXPECT_FALSE(result.has_value());
        EXPECT_EQ(result.error(), IPC::Error::InvalidConnection);
        done = true;
    });
    EXPECT_FALSE(done);

    while (!done)
        RunLoop::current().cycle();

}

INSTANTIATE_TEST_SUITE_P(MessageSenderTest,
    MessageSenderTest,
    testing::Values(ConnectionTestDirection::ServerIsA, ConnectionTestDirection::ClientIsA),
    TestParametersToStringFormatter());

}

==> webkit2gtk_2.48.5.orig.tar.xz: webkitgtk-2.48.5/Tools/TestWebKitAPI/Tests/IPC/MessageSenderTests.cpp <==
/*
 * Copyright (C) 2022 Apple Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "MessageSender.h"

#include "IPCTestUtilities.h"
#include "MessageSenderInlines.h"
#include "Test.h"

namespace TestWebKitAPI {

class MessageSenderTest : public ConnectionTestABBA {
};

namespace {

class SimpleMessageSender : public IPC::MessageSender {
public:
    SimpleMessageSender(IPC::Connection* connection)
        : m_connection(connection)
    {
    }

private:
    IPC::Connection* messageSenderConnection() const { return m_connection; };
    uint64_t messageSenderDestinationID() const { return 0u; };
    IPC::Connection* m_connection;
};

}

// Tests that async reply messages sent to a connection after invalidate()
// will receive the replies as cancelled.
TEST_P(MessageSenderTest, SendAsyncAfterInvalidateCancelsAllAsyncReplies)
{
    ASSERT_TRUE(openBoth());
    b()->invalidate();

    // This works for IPC::Connection.
    {
        HashSet<uint64_t> replies;
        for (uint64_t i = 100u; i < 160u; ++i) {
            b()->sendWithAsyncReply(MockTestMessageWithAsyncReply1 { }, [&, j = i] (uint64_t value) {
                EXPECT_EQ(value, 0u) << j;
                if (!value)
                    replies.add(j);
            }, i);
        }
        while (replies.size() < 60u)
            RunLoop::current().cycle();
    }

    // Same should work via IPC::MessageSender.
    {
        SimpleMessageSender sender { b() };
        HashSet<uint64_t> replies;
        for (uint64_t i = 100u; i < 160u; ++i) {
            sender.sendWithAsyncReply(MockTestMessageWithAsyncReply1 { }, [&, j = i] (uint64_t value) {
                EXPECT_EQ(value, 0u) << j;
                if (!value)
                    replies.add(j);
            }, i);
        }
        while (replies.size() < 60u)
            RunLoop::current().cycle();
    }
}

// Tests that async reply messages sent to a connection after invalidate()
// will receive the error.
TEST_P(MessageSenderTest, SendWithPromisedReplyAfterInvalidateCancelsAllAsyncReplies)
{
    ASSERT_TRUE(openBoth());
    b()->invalidate();

    // This works for IPC::Connection.
    bool done = false;

    b()->sendWithPromisedReply(MockTestMessageWithAsyncReply1 { }, 100)->whenSettled(RunLoop::protectedCurrent(), [&] (MockTestMessageWithAsyncReply1::Promise::Result result) {
        EXPECT_FALSE(result.has_value());
        EXPECT_EQ(result.error(), IPC::Error::InvalidConnection);
        done = true;
    });

    while (!done)
        RunLoop::current().cycle();

    // Same should work via IPC::MessageSender.
    done = false;
    SimpleMessageSender sender { b() };

    sender.sendWithPromisedReply(MockTestMessageWithAsyncReply1 { }, 100)->whenSettled(RunLoop::protectedCurrent(), [&] (MockTestMessageWithAsyncReply1::Promise::Result result) {
        EXPECT_FALSE(result.has_value());
        EXPECT_EQ(result.error(), IPC::Error::InvalidConnection);
        done = true;
    });

    while (!done)
        RunLoop::current().cycle();
}

// Tests that async reply messages sent to a connection after invalidate()
// will receive the error asynchronously.
TEST_P(MessageSenderTest, SendAsyncWithErrorAsynchronousCallback)
{
    ASSERT_TRUE(openBoth());
    b()->invalidate();


    bool done = false;
    b()->sendWithAsyncReply(MockTestMessageWithAsyncReply1 { }, [&] (uint64_t value) {
        // Value is default constructed here, and no error is actually returned.
        done = true;
    }, 100);
    EXPECT_FALSE(done);

    while (!done)
        RunLoop::current().cycle();

    done = false;
    b()->sendWithPromisedReply(MockTestMessageWithAsyncReply1 { }, 100)->whenSettled(RunLoop::protectedCurrent(), [&] (MockTestMessageWithAsyncReply1::Promise::Result result) {
        EXPECT_FALSE(result.has_value());
        EXPECT_EQ(result.error(), IPC::Error::InvalidConnection);
        done = true;
    });
    EXPECT_FALSE(done);

    while (!done)
        RunLoop::current().cycle();

}

INSTANTIATE_TEST_SUITE_P(MessageSenderTest,
    MessageSenderTest,
    testing::Values(ConnectionTestDirection::ServerIsA, ConnectionTestDirection::ClientIsA),
    TestParametersToStringFormatter());

}
